; BCE.ASM
; Ben's Command Extender
; loads and unloads BCE
; 5 Dec 97
; 12 Dec 97 improved "already loaded" check
; blm
;

%INCLUDE "bce.inc"

BITS 16

ORG 0100h
SECTION .text

start:
	jmp     init


; ___________________________________________________________________________
; resident code

; int_2Fh
; the redirected interrupt 
int_2Fh:
	pushf
	push    ax
	
	cmp     ax, 0AE01h
	je      .execute
	cmp     ax, 0AE00h
	jne     .wrong_int
	call    confirm
	cmp     al, TRUE
	jne     .wrong_int
	pop     ax
	mov     al, TRUE
	popf
	iret
	
.wrong_int:
	pop     ax
	popf
	jmp     far [cs:org_2Fh]

.execute:
	call    arg_init
	call    [cs:proc_ptr]
	call	set_err
	mov     byte [ds:si], 00h
	pop     ax
	popf    
	iret
	
	
	
; confirm
; check if the command is a valid BCE.COM command
; returns status (TRUE or FALSE) in ax
confirm:
	push    cx
	push    di
	push    si
	push    es
	
	push    cs
	pop     es
	
	mov     al, [ds:si]
	cmp     al, 03h
	jne     .not_three
	mov     di, three_commands
	mov     cx, THREE_COUNT
	mov     word [cs:command_length], 03h
	jmp     .check_command

.not_three:
	mov     al, [ds:si]
	cmp     al, 04h
	jne     .not_four
	mov     di, four_commands
	mov     cx, FOUR_COUNT
	mov     word [cs:command_length], 04h
	jmp     .check_command

.not_four:
	mov     al, [ds:si]
	cmp     al, 05h
	jne     .not_five
	mov     di, five_commands
	mov     cx, FIVE_COUNT
	mov     word [cs:command_length], 05h
	jmp     .check_command

.not_five
	xor     al, al
	pop     es
	pop     si
	pop     di
	pop     cx
	ret
	
.check_command:
	cld
	inc     si
	xor     al, al
	
.next_command:
	push    cx
	push    di
	push    si
	mov     cx, [cs:command_length]
	repe    cmpsb
	jne     .not_found
	mov     al, TRUE
.not_found:
	pop     si
	pop     di
	pop     cx
	cmp     al, TRUE
	je      .found
	add     di, [cs:command_length]
	add     di, 2
	loop    .next_command

.found:
	add     di, [cs:command_length]
	mov     si, [cs:di]
	mov     [cs:proc_ptr], si
	
	pop     es
	pop     si
	pop     di
	pop     cx
	ret
	
end_tsr_code:   

%INCLUDE "string.asm"
%INCLUDE "input.asm"
%INCLUDE "output.asm"
%INCLUDE "number.asm"
%INCLUDE "rand.asm"
%INCLUDE "newline.asm"
%INCLUDE "arg.asm"
%INCLUDE "math.asm"
%INCLUDE "char.asm"
%INCLUDE "file.asm"
%INCLUDE "error.asm"



;----------------------------------------------------------------------------
; Resident data

; TSR specific
org_2Fh         dd 00000000h                    ; original vector
id_code         dw ID_WORD                      ; already installed id

; used by confirm and int_2Fh
command_length  dw 0000h                        ; length of BCE.COM command
proc_ptr        dw 0000h                        ; pointer to BCE.COM proc

; BCE.COM commands and vectors
three_commands  db 'ADD'                        ; name
				dw addition                     ; vector
				db 'BIN'                        ; yadda...
				dw binary                       ; yadda, yadda
				db 'EKO' 
				dw eko
				db 'DEC'                        ; all BCE.COM commands
				dw decimal                      ; three chars long
				db 'DIV'
				dw division
				db 'HEX'
				dw hexadecimal
				db 'MOD'
				dw modulous
				db 'MUL'
				dw multiplication
				db 'OCT'
				dw octal
				db 'SUB'
				dw subtraction
THREE_COUNT equ 10

four_commands   db 'RAND'                       ; all BCE.COM commands 
				dw  rand                        ; four chars long
				db 'LFON'
				dw  lfon
				db 'EKOF'
				dw  ekof
FOUR_COUNT equ 3

five_commands   db 'COUNT'
				dw count
				db 'FDATE'
				dw  fdate
				db 'FTIME'
				dw ftime
				db 'INPUT'
				dw input
				db 'LCASE'
				dw lcase
				db 'LFOFF'                      ; all BCE.COM commands
				dw lfoff                        ; five chars long
				db 'NOCHR'
				dw nochr
				db 'NOCTR'
				dw noctr
				db 'NOEXT'
				dw noext
				db 'RANDM'
				dw randm
				db 'RANDR'
				dw randr
				db 'UCASE'
				dw ucase
				db 'XPAND'
				dw xpand
FIVE_COUNT equ 13



;----------------------------------------------------------------------------
; Non resident code

init:   
		mov     si,80h                      ;pointer to command line

; check for unload switch first
.check_unload:
		inc     si
		cmp     word [si],'/u'
		je      unload
		cmp     word [si],'/U'
		je      unload
		cmp     byte [si],CR
		jne     .check_unload

; check for previous install
		mov     byte [errorlevel], PREVIOUS_INSTALL
		mov     dx, previous
		mov		si, test_comm
		mov		ax, 0AE00h
		int		2Fh
		and		al, al
		jz		.not_previous
		jmp     quit
.not_previous:

; get interrupt address
		mov     ax,352Fh 
		int     21h
		
; save original interrupt address
		mov     [org_2Fh],bx
		mov     [org_2Fh+2],es
		
; point vector to new routine
		mov     dx, int_2Fh 
		mov     ax,252Fh
		int     21h

; display loaded message
		mov     dx, loaded
		mov     ah,09h
		int     21h
		
; TSR       
		mov     dx,((init-start)+10Fh)/10h ;number of paragraphs to reserve
		mov     ax, 3100h               
		int     21h

;----------------------------------------------------------------------------
; Unload TSR

unload:

; compare resident interrupt code to code above
	mov     byte [errorlevel], UNLOAD_FAIL1
	mov     ax,352Fh
	int     21h
	mov     di,bx
	mov     si, int_2Fh 
	mov     cx,end_tsr_code-int_2Fh ; length of code
	repe    cmpsb                   
	jne     .fail
		
; restore original interrupt
	push    ds         
	lds     dx,[es:org_2Fh] 
	mov     ax,252Fh
	int     21h
	pop     ds                  
   
	mov     byte [errorlevel], UNLOAD_FAIL2
	push    es
	mov     bx,es                   
	mov     ax,[es:2Ch]             
	push    ax                      
	dec     ax                      
	mov     es,ax                   
	cmp     bx,[es:01h]             
	jne     .fail
	pop     es                      
	mov     byte [errorlevel], UNLOAD_FAIL3
	mov     ah,49h                  
	int     21h
	jc      .fail                    
	pop     es                      
	
	mov     dx, unloded
	mov     byte [errorlevel], SUCCESS
	mov     ah,49h                  
	int     21h
	jnc     quit                    
	mov     byte [errorlevel], UNLOAD_FAIL4
	
.fail:
	mov     dx, no_unld


; quit with message and errorlevel
quit:   
	mov     ah,09h                  
	int     21h
	mov     ah,4Ch                  
	mov     al, [errorlevel]
	int     21h
	
	
	
;----------------------------------------------------------------------------
; non resident data

test_comm	db 3, 'EKO'
previous    db 'ERROR:  BCE.COM IS already loaded!', CR, LF, '$'
no_unld     db 'ERROR:  BCE.COM can NOT be unloaded!', CR, LF, '$'
loaded      db 'BCE.COM Version 0.01', CR, LF
			db 'Public Domain 1997 blm', CR, LF
			db 'BCE.COM is loaded.', CR, LF , '$'
unloded     db 'BCE.COM has been unloaded.', CR, LF, '$'
errorlevel  db 0
